import java.util.*;

/**
 * Write a description of class FileConverter here.
 * 
 * @author (your name) 
 * @version (a version number or a date)
 */
public class FileConverter
{
    private SimpleDecoder decoder;
    private SimpleEncoder encoder;
    private Compressor compressor;
    private FileHandler fileHandler;
    
    /**
     * Constructor for objects of class FileConverter
     */
    public FileConverter()
    {
        encoder = new SimpleEncoder();
        decoder = new SimpleDecoder();
        compressor = new Compressor();
        fileHandler = new FileHandler();
    }

    /**
     * Encode the specified file into 'zup' format. The file must exist, and
     * it must be a text file with suffix ".txt". The filename may or may not
     * include the suffix. If not, the ".txt" suffix is automatically appended.
     * The method creates an encoded file with the same base name and ".zup"
     * suffix.
     * 
     * @param filename  The file name of the file to be encoded.
     */
    public void encode(String filename)
    {
        String basename = getBaseName(filename, ".txt");
        List text = fileHandler.readFile(basename + ".txt");
        List encoded = encoder.encode(text);
        fileHandler.writeFile(basename + ".zup", encoded);
    }

    /**
     * Decode the specified file from 'zup' to 'txt' format. The file must 
     * exist, and it must be a zup-encoded file with suffix ".zup". The 
     * filename may or may not include the suffix. If not, the ".zup" suffix 
     * is automatically appended. The method creates a decoded file with 
     * the same base name and ".txt" suffix.
     * 
     * @param filename  The file name of the file to be decoded.
     */
    public void decode(String filename)
    {
        String basename = getBaseName(filename, ".zup");
        List encoded = fileHandler.readFile(basename + ".zup");
        List text = decoder.decode(encoded);
        fileHandler.writeFile(basename + ".txt", text);
    }

    /**
     * 
     * @param filename  The file name of the file to be encoded.
     */
    public void compress(String filename)
    {
        String basename = getBaseName(filename, ".txt");
        List text = fileHandler.readFile(basename + ".txt");
        List encoded = encoder.encode(text);
        List compressed = compressor.compress(encoded);
        fileHandler.writeFile(basename + ".czp", compressed);
    }

    /**
     * 
     * @param filename  The file name of the file to be decoded.
     */
    public void uncompress(String filename)
    {
        String basename = getBaseName(filename, ".czp");
        List compressed = fileHandler.readFile(basename + ".czp");
        List encoded = compressor.uncompress(compressed);
        List text = decoder.decode(encoded);
        fileHandler.writeFile(basename + ".txt", text);
    }

    /**
     * Show the content of the requested file in the text terminal.
     * The file must exist, and it must be a text file with suffix ".txt". 
     * The filename may or may not include the suffix. If not, the ".txt" 
     * suffix is automatically appended.
     * 
     * @param filename  The file name of the file to be displayed.
     */
    public void showFile(String filename)
    {
        String basename = getBaseName(filename, ".txt");
        List text = fileHandler.readFile(basename + ".txt");
        showText(text);
    }
    
    /**
     * Decode and show the content of the requested file in the text terminal.
     * The file must exist, and it must be a zup file with suffix ".zup". 
     * The filename may or may not include the suffix. If not, the ".zup" 
     * suffix is automatically appended. The file is decoded and displayed.
     * 
     * @param filename  The file name of the file to be displayed.
     */
    public void decodeAndShowFile(String filename)
    {
        String basename = getBaseName(filename, ".zup");
        List encoded = fileHandler.readFile(basename + ".zup");
        List text = decoder.decode(encoded);
        showText(text);
    }

    /**
     * Uncompress and show the content of the requested file in the text terminal.
     * The file must exist, and it must be a czp file with suffix ".czp". 
     * The filename may or may not include the suffix. If not, the ".czp" 
     * suffix is automatically appended. The file is uncompressed, decoded and 
     * displayed.
     * 
     * @param filename  The file name of the czp file to be displayed.
     */
    public void uncompressAndShowFile(String filename)
    {
        String basename = getBaseName(filename, ".czp");
        List compressed = fileHandler.readFile(basename + ".czp");
        List encoded = compressor.uncompress(compressed);
        List text = decoder.decode(encoded);
        showText(text);
    }
    /**
     * Display the text in the specified list on the terminal.
     */
    private void showText(List text)
    {
        for(Iterator it = text.iterator(); it.hasNext(); ) {
            char ch = ((Character)it.next()).charValue();
            System.out.print(ch);
        }
    }
    
    /**
     * Get the base name for a given file name. That is: if the name
     * has the indicated suffix, remove it, otherwise leave it unchanged.
     */
    private String getBaseName(String filename, String suffix)
    {
        if(filename.endsWith(suffix)) {
            return filename.substring(0, filename.length()-suffix.length());
        }
        else {
            return filename;
        }
    }
}
